from dj_audit import settings
from dj_audit.models import AuditLog

file_extensions = ['.svg', '.js', '.css', '.png', '.jpg', '.ico', ]


class AuditMiddleware:

    def __init__(self, get_response) -> None:
        self.get_response = get_response

    def __call__(self, request, *args, **kwargs):
        response = self.process_response(request)
        return response

    def process_response(self, request, response=None):
        response = self.get_response(request)
        static_url = getattr(settings, 'STATIC_URL', None)
        if static_url and static_url in request.path_info:
            return response

        media_url = getattr(settings, 'MEDIA_URL', None)
        if media_url and media_url in request.path_info:
            return response

        for ext in file_extensions:
            if ext in request.path_info:
                return response

        if settings.DISABLE_AUDIT_LOG:
            return response

        if not request.method in ('HEAD', 'OPTIONS', 'TRACE'):
            if hasattr(request, 'user') and request.user.is_authenticated:
                user = request.user
            else:
                user = None

            log_data = {
                'user_agent': request.META.get('HTTP_USER_AGENT', ''),
                'ip_address': request.META.get('REMOTE_ADDR', ''),
                'host_name': request.META.get('REMOTE_HOST', ''),
                'user': user,
                'content_type': request.META.get('CONTENT_TYPE', ''),
                'query_string': request.META.get('QUERY_STRING', ''),
                'http_method': request.method,
                'http_referer': request.META.get('HTTP_REFERER', ''),
                'path_info': request.path_info,
                'post_data': request.POST.dict(),
                'response_status_code': response.status_code,
                'response_reason_phrase': response.reason_phrase,
                'response_body': response.content.decode('utf-8'),
            }

            AuditLog.objects.create(**log_data)

        return response



from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth import get_user_model
from django.db.models import Count, Q
from django.http.response import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse

from dj_audit.models import AuditLog
from dj_audit.settings import (API_BASE_URL, AUDIT_LOG_TEMPLATE,
                               REQUEST_STATUS_TEMPLATE)


@staff_member_required
def index_page(request):
    return HttpResponseRedirect(reverse('dj_audit_audit_log_page'))


@staff_member_required
def audit_log_page(request):
    """ List audit logs """
    query_filter = {}
    user_name = request.GET.get('user_name', None)
    if user_name:
        query_filter['user__username'] = user_name
    #
    audit_logs = AuditLog.objects.filter(**query_filter)
    context = {
        'users': get_user_model().objects.all(),
        'audit_logs': audit_logs
    }
    return render(request, AUDIT_LOG_TEMPLATE, context)


@staff_member_required
def request_status_page(request):
    """ Request Status page """

    audit_log = AuditLog.objects.filter(path_info__startswith=API_BASE_URL)
    total_success = audit_log.filter(response_status_code=200).count()
    total_not_found = audit_log.filter(response_status_code=404).count()
    total_failed = audit_log.exclude(response_status_code=200).exclude(
        response_status_code=404).count()

    no_success = Count('id', filter=Q(response_status_code=200))
    no_failed = Count('id', exclude=(
        Q(response_status_code=200) | Q(response_status_code=404)))
    no_not_found = Count('id', filter=Q(response_status_code=404))

    api_break_down = audit_log.values('path_info').\
        annotate(no_success=no_success).annotate(
            no_failed=no_failed).annotate(no_not_found=no_not_found)

    context = {
        'total_success': total_success,
        'total_not_found': total_not_found,
        'total_failed': total_failed,
        'api_break_down': api_break_down
    }
    return render(request, REQUEST_STATUS_TEMPLATE, context)



# Generated by Django 3.2.8 on 2021-10-22 10:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dj_audit', '0002_auditlog_post_data'),
    ]

    operations = [
        migrations.AddField(
            model_name='auditlog',
            name='host_name',
            field=models.CharField(
                default='', max_length=200, verbose_name='HostName'),
        ),
    ]



from django.contrib import admin
from django.urls import path
from django.urls.conf import include

urlpatterns = [
    path("admin/", admin.site.urls),
    path("dj-audit/", include('dj_audit.urls'))
]



from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-h+ttxnre-*7+0aog0*am@59lwa6deeo9t#@1c(w$h)^fx+e_-v'

DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'dj_audit'
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'dj_audit.middleware.AuditMiddleware'
]

ROOT_URLCONF = 'dj_audit.test_urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# DATABASES = {
#     'default': {
#         'ENGINE': 'django.db.backends.sqlite3',
#         'NAME': ":memory:",
#     }
# }

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_URL = '/static/'

# STATIC_ROOT = os.path.join(BASE_DIR, 'static')



from django.urls import path

from .views import audit_log_page, index_page, request_status_page

urlpatterns = [
    path("", index_page, name="dj_audit_index_page"),
    path("audit-logs/", audit_log_page, name="dj_audit_audit_log_page"),
    path("request-status/", request_status_page,
         name="dj_audit_request_status_page"),
]



from django.core.management.base import BaseCommand

from ...models import AuditLog


class Command(BaseCommand):
    """ clean up management command """

    help = "Cleans up dj-audit AuditLog table"

    def handle(self, **options):
        """
        Removes all Audit log from the database
        """
        print("Starting clean up of dj-audit AuditLog table")

        count, _ = AuditLog.objects.all().delete()

        print(f'Deleted {count} records from dj-audit AuditLog table')



from django.conf import settings as global_settings

DISABLE_AUDIT_LOG = getattr(global_settings, 'DISABLE_AUDIT_LOG', False)

AUDIT_LOG_TEMPLATE = getattr(
    global_settings, 'AUDIT_LOG_TEMPLATE', 'dj_audit/audit_list_page.html')

REQUEST_STATUS_TEMPLATE = getattr(
    global_settings, 'REQUEST_STATUS_TEMPLATE', 'dj_audit/request_status_page.html')

API_BASE_URL = getattr(
    global_settings, 'API_BASE_URL', '/admin/')



# Generated by Django 3.2.8 on 2021-10-22 10:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dj_audit', '0003_auditlog_host_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='auditlog',
            name='response_body',
            field=models.TextField(default=''),
        ),
        migrations.AlterField(
            model_name='auditlog',
            name='host_name',
            field=models.CharField(
                default='', max_length=200, verbose_name='Host Name'),
        ),
    ]



from django.contrib.admin import DateFieldListFilter, ModelAdmin, register

from dj_audit.models import AuditLog


@register(AuditLog)
class AuditLogAdmin(ModelAdmin):
    list_display = (
        'user_agent', 'ip_address', 'user', 'content_type', 'query_string',
        'http_method', 'http_referer', 'path_info', 'request_data', 'post_data',
        'response_status_code', 'response_reason_phrase', 'attempt_time'
    )
    list_filter = (
        'user', 'http_method', 'response_status_code',
        ('attempt_time', DateFieldListFilter)
    )
    search_fields = ('user__username', 'ip_address')



from django.contrib.auth.models import User
from django.http import HttpResponse
from django.test import RequestFactory, TestCase

from dj_audit.middleware import AuditMiddleware
from dj_audit.models import AuditLog


def get_response_empty(request):
    return HttpResponse()


class TestAuditMiddleware(TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.user = User.objects.create(
            username='jacob', email='jacob@…', password='top_secret')

    def test_request_processing(self):
        request = self.factory.get('/')

        response = AuditMiddleware(
            get_response_empty).process_response(request)

        self.assertIsNotNone(response)

        audit_logs_count = AuditLog.objects.all().count()
        self.assertEqual(audit_logs_count, 1)

        request.user = self.user
        response = AuditMiddleware(
            get_response_empty).process_response(request)

        self.assertIsNotNone(response)

        audit_logs_count = AuditLog.objects.all().count()
        self.assertEqual(audit_logs_count, 2)

    def test_get_request_processing(self):
        data = {'q': "Hab"}
        request = self.factory.get('/test/', data=data)

        response = AuditMiddleware(
            get_response_empty).process_response(request)

        self.assertIsNotNone(response)

        audit_logs_count = AuditLog.objects.all().count()
        self.assertEqual(audit_logs_count, 1)

        request.user = self.user
        response = AuditMiddleware(
            get_response_empty).process_response(request)

        self.assertIsNotNone(response)

        audit_logs_count = AuditLog.objects.all().count()
        self.assertEqual(audit_logs_count, 2)

        last_log: AuditLog = AuditLog.objects.first()
        self.assertIsNotNone(last_log)

    def test_post_request_processing(self):
        data = {'name': "Habeeb"}
        request = self.factory.post(
            '/test/', data=data, content_type='application/json')

        response = AuditMiddleware(
            get_response_empty).process_response(request)

        self.assertIsNotNone(response)

        audit_logs_count = AuditLog.objects.all().count()
        self.assertEqual(audit_logs_count, 1)

        request.user = self.user
        response = AuditMiddleware(
            get_response_empty).process_response(request)

        self.assertIsNotNone(response)

        audit_logs_count = AuditLog.objects.all().count()
        self.assertEqual(audit_logs_count, 2)

        last_log: AuditLog = AuditLog.objects.first()
        self.assertIsNotNone(last_log)


class TestAuditView(TestCase):

    def setUp(self) -> None:
        return super().setUp()



# Generated by Django 3.2.8 on 2021-10-22 09:54

from django.db import migrations, models
import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    dependencies = [
        ('dj_audit', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='auditlog',
            name='post_data',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True,
                                                                 verbose_name='Post Data'),
        ),
    ]
